path = require "path"
fs = require "fs"
_ = require "underscore"
connect = require "connect"
cs = require "coffee-script"
stylus = require "stylus"

require "./dk-assets-coffeescript"
require "./dk-assets-stylus"

dk.plugin "assets", ["server"], ->
  mod = {}
  packagedJS = []
  packagedCSS = []
  packageFiles = false
  publicDir = "#{process.cwd()}/public"
  jsDir = "#{publicDir}/js"
  cssDir = "#{publicDir}/css"
  userAssets = []
  compilers = {}

  # PUBLIC METHODS

  mod.config = (opts) ->
    packageFiles = !!opts.package
    if _.include _.keys(opts), "dir"
      publicDir = opts.dir

  mod.run = ->
    wireServer(dk.server)
    dk.template.helpers.add
      includeJavascripts: ->
        scripts = _.map mod.jsFiles(), (url) ->
          "<script type='text/javascript' src='#{url}'></script>"
        scripts.join("\n")
      includeCSS: ->
        sheets = _.map mod.cssFiles(), (url) ->
          "<link rel='stylesheet' href='#{url}' />"
        sheets.join "\n"

  mod.package= (url, fileOrFunc, serve=true) ->
    switch path.extname(url)
      when ".js"
        packagedJS.push([url, fileOrFunc])
      when ".css"
        packagedCSS.push([url, fileOrFunc])
    this.serve(url, fileOrFunc) if serve

  mod.serve = (url, fileOrFunc) ->
    dk.server.get url, (req, res) ->
      switch path.extname(url)
        when ".js"
          res.header "Content-Type", "text/javascript"
        when ".css"
          res.header "Content-Type", "text/css"
        when ".jpg"
          res.header "Content-Type", "image/jpeg"
        when ".png"
          res.header "Content-Type", "image/png"
        when ".gif"
          res.header "Content-Type", "image/gif"
        when ".swf"
          res.header "Content-Type", "application/x-shockwave-flash"
      out = getFile fileOrFunc
      res.send out

  mod.jsFiles = ->
    loadUserAssets()
    if packageFiles
      ["/js/all.js"]
    else
      _.map packagedJS, (tuple) ->
        tuple[0]

  mod.cssFiles = ->
    loadUserAssets()
    if packageFiles
      ["/css/all.css"]
    else
      _.map packagedCSS, (tuple) ->
        tuple[0]

  mod.addCompiler = (suffix, func) ->
    compilers[suffix] = func
  
  # PRIVATE METHODS

  packDir = (path) ->
    fs.readdirSync(path).forEach (fileOrDir) ->
      fullPath = "#{path}/#{fileOrDir}"
      stat = fs.statSync fullPath
      if stat.isDirectory()
        packDir fullPath
      else
        relPath = fullPath.split(publicDir)[1]
        relPath = relPath.replace /\.coffee$/, ".js"
        relPath = relPath.replace /\.stylus$/, ".css"
        userAssets.push [relPath, fullPath]

  loadUserAssets = _.once ->
    while asset = userAssets.shift()
      mod.package asset[0], asset[1]

  wireServer = (server) ->
    if publicDir
      server.use connect.static(publicDir)
    packDir(jsDir) if jsDir
    packDir(cssDir) if cssDir

    server.get "/js/all.js", (req, res) ->
      out = _.reduce(packagedJS, (memo, tuple) ->
        filepath = tuple[1]
        memo += getFile(filepath)
        memo += "\n"
      , "")
      res.header "Content-Type", "text/javascript"
      res.send out

    server.get "/css/all.css", (req, res) ->
      out = _.reduce(packagedCSS, (memo, tuple) ->
        filepath = tuple[1]
        memo += getFile(filepath)
        memo += "\n"
      , "")
      res.header "Content-Type", "text/css"
      res.send out

  compileAsset = (content, suffix) ->
    compiler = compilers[suffix] || (c) -> c
    compiler content

  getFile = (fileOrFunc) ->
    if _.isFunction fileOrFunc
      fileOrFunc()
    else
      suffix = fileOrFunc.split(".").pop()
      content = fs.readFileSync(fileOrFunc).toString()
      compileAsset content, suffix

  return mod
